@@ -23,6 +23,7 @@ from __future__ import absolute_import, division, print_function
 import ast
 import base64
 import contextlib
+import datetime
 import inspect
 import os
 import random as rnd_module
@@ -41,6 +42,7 @@ from hypothesis._settings import (
     PrintSettings,
     Verbosity,
     local_settings,
+    note_deprecation,
     settings as Settings,
 )
 from hypothesis.control import BuildContext
@@ -229,6 +231,7 @@ def is_invalid_test(name, original_argspec, generator_arguments, generator_kwarg
         def wrapped_test(*arguments, **kwargs):
             raise InvalidArgument(message)
 
+        wrapped_test.is_hypothesis_test = True
         return wrapped_test
 
     if not (generator_arguments or generator_kwargs):
@@ -243,12 +246,11 @@ def is_invalid_test(name, original_argspec, generator_arguments, generator_kwarg
         )
 
     if len(generator_arguments) > len(original_argspec.args):
+        args = tuple(generator_arguments)
         return invalid(
-            (
-                "Too many positional arguments for %s() (got %d but"
-                " expected at most %d"
-            )
-            % (name, len(generator_arguments), len(original_argspec.args))
+            "Too many positional arguments for %s() were passed to @given "
+            "- expected at most %d arguments, but got %d %r"
+            % (name, len(original_argspec.args), len(args), args)
         )
 
     if infer in generator_arguments:
@@ -265,16 +267,15 @@ def is_invalid_test(name, original_argspec, generator_arguments, generator_kwarg
         if k not in original_argspec.args + original_argspec.kwonlyargs
     ]
     if extra_kwargs and not original_argspec.varkw:
+        arg = extra_kwargs[0]
         return invalid(
-            "%s() got an unexpected keyword argument %r" % (name, extra_kwargs[0])
+            "%s() got an unexpected keyword argument %r, from `%s=%r` in @given"
+            % (name, arg, arg, generator_kwargs[arg])
         )
     for a in original_argspec.args:
         if isinstance(a, list):  # pragma: no cover
             return invalid(
-                (
-                    "Cannot decorate function %s() because it has "
-                    "destructuring arguments"
-                )
+                "Cannot decorate function %s() because it has destructuring arguments"
                 % (name,)
             )
     if original_argspec.defaults or original_argspec.kwonlydefaults:
@@ -397,6 +398,23 @@ def process_arguments_to_given(
     return arguments, kwargs, test_runner, search_strategy
 
 
+def run_once(fn):
+    """Wraps a no-args function so that its outcome is cached.
+    We use this for calculating various lists of exceptions
+    the first time we use them."""
+    result = [None]
+
+    def run():
+        if result[0] is None:
+            result[0] = fn()
+            assert result[0] is not None
+        return result[0]
+
+    run.__name__ = fn.__name__
+    return run
+
+
+@run_once
 def skip_exceptions_to_reraise():
     """Return a tuple of exceptions meaning 'skip this test', to re-raise.
 
@@ -406,7 +424,7 @@ def skip_exceptions_to_reraise():
     import unittest
 
     # This is a set because nose may simply re-export unittest.SkipTest
-    exceptions = set([unittest.SkipTest])
+    exceptions = {unittest.SkipTest}
 
     try:  # pragma: no cover
         from unittest2 import SkipTest
@@ -432,9 +450,7 @@ def skip_exceptions_to_reraise():
     return tuple(sorted(exceptions, key=str))
 
 
-EXCEPTIONS_TO_RERAISE = skip_exceptions_to_reraise()
-
-
+@run_once
 def failure_exceptions_to_catch():
     """Return a tuple of exceptions meaning 'this test has failed', to catch.
 
@@ -451,9 +467,6 @@ def failure_exceptions_to_catch():
     return tuple(exceptions)
 
 
-EXCEPTIONS_TO_FAIL = failure_exceptions_to_catch()
-
-
 def new_given_argspec(original_argspec, generator_kwargs):
     """Make an updated argspec for the wrapped test."""
     new_args = [a for a in original_argspec.args if a not in generator_kwargs]
@@ -517,11 +530,13 @@ class StateForActualGivenExecution(object):
                 result = self.test(*args, **kwargs)
                 finish = benchmark_time()
                 internal_draw_time = sum(data.draw_times[initial_draws:])
-                runtime = (finish - start - internal_draw_time) * 1000
+                runtime = datetime.timedelta(
+                    seconds=finish - start - internal_draw_time
+                )
                 self.__test_runtime = runtime
                 current_deadline = self.settings.deadline
                 if not is_final:
-                    current_deadline *= 1.25
+                    current_deadline = (current_deadline // 4) * 5
                 if runtime >= current_deadline:
                     raise DeadlineExceeded(runtime, self.settings.deadline)
                 return result
@@ -530,8 +545,8 @@ class StateForActualGivenExecution(object):
             if not hasattr(data, "can_reproduce_example_from_repr"):
                 data.can_reproduce_example_from_repr = True
             with local_settings(self.settings):
-                with BuildContext(data, is_final=is_final):
-                    with deterministic_PRNG():
+                with deterministic_PRNG():
+                    with BuildContext(data, is_final=is_final):
                         args, kwargs = data.draw(self.search_strategy)
                         if expected_failure is not None:
                             text_repr[0] = arg_string(test, args, kwargs)
@@ -569,7 +584,11 @@ class StateForActualGivenExecution(object):
                         "variability in your test timings, consider turning "
                         "deadlines off for this test by setting deadline=None."
                     )
-                    % (exception.runtime, self.settings.deadline, self.__test_runtime)
+                    % (
+                        exception.runtime.total_seconds() * 1000,
+                        self.settings.deadline.total_seconds() * 1000,
+                        self.__test_runtime.total_seconds() * 1000,
+                    )
                 )
             else:
                 report("Failed to reproduce exception. Expected: \n" + traceback)
@@ -601,9 +620,9 @@ class StateForActualGivenExecution(object):
             HypothesisDeprecationWarning,
             FailedHealthCheck,
             StopTest,
-        ) + EXCEPTIONS_TO_RERAISE:
+        ) + skip_exceptions_to_reraise():
             raise
-        except EXCEPTIONS_TO_FAIL as e:
+        except failure_exceptions_to_catch() as e:
             escalate_hypothesis_internal_error()
             if data.frozen:
                 # This can happen if an error occurred in a finally
@@ -662,6 +681,8 @@ class StateForActualGivenExecution(object):
 
         if not self.falsifying_examples:
             return
+        elif not self.settings.report_multiple_bugs:
+            del self.falsifying_examples[:-1]
 
         self.failed_normally = True
 
@@ -1034,6 +1055,11 @@ def find(
     # type: (...) -> Any
     """Returns the minimal example from the given strategy ``specifier`` that
     matches the predicate function ``condition``."""
+    note_deprecation(
+        "`find(s, f)` is deprecated, because it is rarely used but takes "
+        "ongoing work to maintain as we upgrade other parts of Hypothesis.",
+        since="2019-07-11",
+    )
     if settings is None:
         settings = Settings(max_examples=2000)
     settings = Settings(settings, suppress_health_check=HealthCheck.all())
@@ -1056,15 +1082,15 @@ def find(
     last_repr = [None]
 
     def template_condition(data):
-        with BuildContext(data):
-            try:
-                data.is_find = True
-                with deterministic_PRNG():
+        with deterministic_PRNG():
+            with BuildContext(data):
+                try:
+                    data.is_find = True
                     result = data.draw(search)
                     data.note(result)
                     success = condition(result)
-            except UnsatisfiedAssumption:
-                data.mark_invalid()
+                except UnsatisfiedAssumption:
+                    data.mark_invalid()
 
         if success:
             successful_examples[0] += 1
@@ -1095,8 +1121,8 @@ def find(
         data = ConjectureData.for_buffer(
             list(runner.interesting_examples.values())[0].buffer
         )
-        with BuildContext(data):
-            with deterministic_PRNG():
+        with deterministic_PRNG():
+            with BuildContext(data):
                 return data.draw(search)
     if runner.valid_examples == 0 and (runner.exit_reason != ExitReason.finished):
         raise Unsatisfiable(
